var hexArrWithSpaceRegex = /^[a-f 0-9]*$/i;
var hexArrWithoutSpaceRegex = /^[a-f0-9]*$/i;

var positiveIntRegex = /^[0-9]*$/i;

function copy(obj) {
 var copyObj;

 if(typeof obj === 'object') {

   if(isNaN(obj.length)) {
     copyObj = {};
     for(var each in obj) {
       if(typeof obj[each] === 'object') {
         copyObj[each] = copy(obj[each]);
       } else {
         copyObj[each] = obj[each];
       }
     }
   } else {
     copyObj = [];
     for(var i=0; i<obj.length; i++) {
       if(typeof obj[i] === 'object') {
         copyObj[i] = copy(obj[i]);
       } else {
         copyObj[i] = obj[i];
       }
     }
   }

   return copyObj;
 }

 return obj;
}

function prependZero(input, length) {
  while(input.length < length) {
      input = '0' + input;
  }
  return input;
}

function getDateAndTime(dateAndTime) {
  var year = dateAndTime.getFullYear();
  var month = prependZero((dateAndTime.getMonth() + 1).toString(), 2);
  var date = prependZero(dateAndTime.getDate().toString(), 2);

  return year + '-' + month + '-' + date + ' ' + dateAndTime.toLocaleTimeString().replace(/:/g, '-');
}

function validateJSON(obj) {
  try {
    JSON.parse(obj);
    return true;
  } catch(e) {
    return false;
  }
}

/*
 *  Command Sequence
 */

function getSeqFileValidity(sequence) {
  return typeof sequence.path === 'string' && sequence.path.length > 0 && typeof sequence.title === 'string' && sequence.title.length > 0;
}

/*
 *  Graph
 */

const getMinMaxIndex = dataArr => {
  // Find the min and max index in each bin. Used inside sampleData function
  let index;
  let data;
  let min = dataArr[0];
  let max = dataArr[0];
  let minIndex = 0;
  let maxIndex = 0;

  if (dataArr.length === 2) {
    minIndex = 0;
    maxIndex = 1;
    return [minIndex, maxIndex];
  }
  for (index in dataArr) {
    data = dataArr[index];
    if (min > data) {
      min = data;
      minIndex = parseInt(index, 10);
    }
    if (max < data) {
      max = data;
      maxIndex = parseInt(index, 10);
    }
  }
  return minIndex < maxIndex ? [minIndex, maxIndex] : [maxIndex, minIndex];
};

const sampleData = (data, domain, width) => {
  // Function to Down Sample the graph data based on the width
  let min = Math.floor(domain[0]);

  let max = Math.ceil(domain[1]);
  let binsPerPx;
  let reducedData = [];
  let firstPoint;
  let toProcess;
  let i;
  let zoomStatus = true;

  let zoomData;
  for (i in data) {
    zoomData = data[i];
    if (
      zoomData.x.length > 1 &&
      min === Math.floor(+zoomData.x[0]) &&
      max === Math.ceil(+zoomData.x[zoomData.x.length - 1])
    ) {
      zoomStatus = false;
    }
  }
  if (!zoomStatus) {
    reducedData = data.slice(0, data.length);
  } else {
    // Condidering only the points that are displayed after zoom
    for (i in data) {
      firstPoint = undefined;
      reducedData.push({
        x: [],
        y: []
      });
      for (let ii in data[i].x) {
        // Assuming x and y will be of same length array
        let currData = {
          x: data[i].x,
          y: data[i].y
        };
        if (currData.x[ii] >= min && currData.x[ii] <= max) {
          if (firstPoint === undefined) {
            firstPoint = ii - 1;
            if (currData.x[ii - 1] && currData.y[ii - 1]) {
              reducedData[i].x.push(currData.x[ii - 1]);
              reducedData[i].y.push(currData.y[ii - 1]);
            }
          }
          reducedData[i].x.push(currData.x[ii]);
          reducedData[i].y.push(currData.y[ii]);
        } else if (currData.x[ii] > max) {
          if (reducedData[i].x.length !== 0) {
            reducedData[i].x.push(currData.x[ii]);
            reducedData[i].y.push(currData.y[ii]);
          }
          break;
        }
      }
    }
  }
  for (i in reducedData) {
    // Down sampling the data
    toProcess = reducedData[i];
    binsPerPx = Math.floor(toProcess.x.length / width);
    if (toProcess.x.length > width * 2) {
      let temp = {
        x: [],
        y: []
      };
      temp.x.push(toProcess.x[0]);
      temp.y.push(toProcess.y[0]);
      for (let k = 0; k < toProcess.x.length; k += binsPerPx) {
        [min, max] = getMinMaxIndex(toProcess.y.slice(k, k + binsPerPx));
        if (k + min !== 0) {
          temp.x.push(toProcess.x[k + min]);
          temp.y.push(toProcess.y[k + min]);
        }
        if (k + max !== toProcess.x.length - 1) {
          temp.x.push(toProcess.x[k + max]);
          temp.y.push(toProcess.y[k + max]);
        }
      }
      temp.x.push(toProcess.x[toProcess.x.length - 1]);
      temp.y.push(toProcess.y[toProcess.y.length - 1]);
      reducedData[i] = temp;
    }
  }
  return reducedData;
};

/*
 *  App Config Editor
 */

 function findPageName(pages, pageName) {
   var found = false;

   for(var i=0; i<pages.length; i++) {
    if(pages[i].pageTitle === pageName) {
      found = true; 
    }
   }

   return found;
 }

 function findGroupName(controlGroup, groupName) {
   var found = false;

   for(var i=0; i<controlGroup.length; i++) {
    if(controlGroup[i].groupTitle === groupName) {
      found = true; 
    }
   }

   return found;
 }

 function findControlName(control, controlName) {
   var found = false;

   for(var i=0; i<control.length; i++) {
    if(control[i].title === controlName) {
      found = true; 
    }
   }

   return found;
 }
